@@ -51,6 +51,7 @@ class Intervention:
             do_plot = True
         self.do_plot = do_plot
         self.days = []
+        self.initialized = False
         return
 
 
@@ -84,6 +85,7 @@ class Intervention:
         Initialize intervention -- this is used to make modifications to the intervention
         that can't be done until after the sim is created.
         '''
+        self.initialized = True
         return
 
 
@@ -121,9 +123,9 @@ class Intervention:
         if self.do_plot or self.do_plot is None:
             if ax is None:
                 ax = pl.gca()
-            ylims = ax.get_ylim()
             for day in self.days:
-                pl.plot([day]*2, ylims, '--', c=[0,0,0])
+                if day is not None:
+                    ax.axvline(day, linestyle='--', c=[0,0,0])
         return
 
 
@@ -234,6 +236,7 @@ class sequence(Intervention):
     def initialize(self, sim):
         ''' Fix the dates '''
         self.days = [sim.day(day) for day in self.days]
+        self.initialized = True
         return
 
 
@@ -293,6 +296,7 @@ class change_beta(Intervention):
                 self.orig_betas['overall'] = sim['beta']
             else:
                 self.orig_betas[lkey] = sim['beta_layer'][lkey]
+        self.initialized = True
         return
 
 
@@ -312,14 +316,6 @@ class change_beta(Intervention):
         return
 
 
-    def plot(self, sim, ax):
-        ''' Plot vertical lines for when changes in beta '''
-        ylims = ax.get_ylim()
-        for day in self.days:
-            pl.plot([day]*2, ylims, '--', c=[0,0,0])
-        return
-
-
 class clip_edges(Intervention):
     '''
     Isolate contacts by moving them from the simulation to this intervention.
@@ -350,9 +346,10 @@ class clip_edges(Intervention):
 
     def initialize(self, sim):
         ''' Fix the dates '''
-        self.start_day = sim.day(self.start_day)
-        self.end_day   = sim.day(self.end_day)
-        self.days      = [self.start_day, self.end_day]
+        self.start_day   = sim.day(self.start_day)
+        self.end_day     = sim.day(self.end_day)
+        self.days        = [self.start_day, self.end_day]
+        self.initialized = True
         return
 
 
@@ -397,6 +394,7 @@ class clip_edges(Intervention):
                 if verbose:
                     print(f'Remaining contacts: {sim.people.contacts[lkey]}')
 
+        # At the end, move back
         if sim.t == self.end_day:
             if verbose:
                 print(f'Before:\n{sim.people.contacts}')
@@ -405,16 +403,13 @@ class clip_edges(Intervention):
                 print(f'After:\n{sim.people.contacts}')
             self.contacts = None # Reset to save memory
 
+        # If no end day is specified, ensure they get deleted
+        if sim.t == sim.tvec[-1]:
+            self.contacts = None # Reset to save memory
+
         return
 
 
-    def plot(self, sim, ax):
-        ''' Plot vertical lines for when changes in beta '''
-        ylims = ax.get_ylim()
-        for day in self.days:
-            pl.plot([day]*2, ylims, '--', c=[0,0,0])
-        return
-
 
 #%% Testing interventions
 
@@ -460,9 +455,10 @@ class test_num(Intervention):
 
     def initialize(self, sim):
         ''' Fix the dates '''
-        self.start_day = sim.day(self.start_day)
-        self.end_day   = sim.day(self.end_day)
-        self.days      = [self.start_day, self.end_day]
+        self.start_day   = sim.day(self.start_day)
+        self.end_day     = sim.day(self.end_day)
+        self.days        = [self.start_day, self.end_day]
+        self.initialized = True
         return
 
 
@@ -547,9 +543,10 @@ class test_prob(Intervention):
 
     def initialize(self, sim):
         ''' Fix the dates '''
-        self.start_day = sim.day(self.start_day)
-        self.end_day   = sim.day(self.end_day)
-        self.days      = [self.start_day, self.end_day]
+        self.start_day   = sim.day(self.start_day)
+        self.end_day     = sim.day(self.end_day)
+        self.days        = [self.start_day, self.end_day]
+        self.initialized = True
         return
 
 
@@ -616,6 +613,7 @@ class contact_tracing(Intervention):
         if sc.isnumber(self.trace_time):
             val = self.trace_time
             self.trace_time = {k:val for k in sim.people.layer_keys()}
+        self.initialized = True
         return
 
 
