@@ -1,5 +1,5 @@
 '''
-Functions for running multiple Covasim runs.
+Functions and classes for running multiple Covasim runs.
 '''
 
 #%% Imports
@@ -7,17 +7,15 @@ import numpy as np
 import pylab as pl
 import pandas as pd
 import sciris as sc
-import datetime as dt
 from collections import defaultdict
-import matplotlib.ticker as ticker
 from . import misc as cvm
-from . import defaults as cvd
 from . import base as cvb
 from . import sim as cvs
+from . import plotting as cvplt
 
 
 # Specify all externally visible functions this file defines
-__all__ = ['make_metapars', 'Scenarios', 'single_run', 'multi_run']
+__all__ = ['make_metapars', 'MultiSim', 'Scenarios', 'single_run', 'multi_run']
 
 
 
@@ -34,6 +32,277 @@ def make_metapars():
     return metapars
 
 
+class MultiSim(sc.prettyobj):
+    '''
+    Class for running multiple copies of a simulation. The parameter n_runs
+    controls how many copies of the simulation there will be, if a list of sims
+    is not provided.
+
+    Args:
+        sims (Sim or list): a single sim or a list of sims
+        base_sim (Sim): the sim used for shared properties; if not supplied, the first of the sims provided
+        quantiles (dict): the quantiles to use with reduce(), e.g. [0.1, 0.9] or {'low:'0.1, 'high':0.9}
+        kwargs (dict): stored in run_args and passed to run()
+
+    Returns:
+        msim: a MultiSim object
+
+    **Examples**::
+
+        sim = cv.Sim() # Create the sim
+        msim = cv.MultiSim(sim, n_runs=5) # Create the multisim
+        msim.run() # Run them in parallel
+        msim.combine() # Combine into one sim
+        msim.plot() # Plot results
+
+        sim = cv.Sim() # Create the sim
+        msim = cv.MultiSim(sim, n_runs=11, noise=0.1) # Set up a multisim with noise
+        msim.run() # Run
+        msim.reduce() # Compute statistics
+        msim.plot() # Plot
+
+        sims = [cv.Sim(beta=0.015*(1+0.02*i)) for i in range(5)] # Create sims
+        for sim in sims: sim.run() # Run sims in serial
+        msim = cv.MultiSim(sims) # Convert to multisim
+        msim.plot() # Plot as single sim
+    '''
+
+    def __init__(self, sims=None, base_sim=None, quantiles=None, **kwargs):
+
+        # Handle inputs
+        if base_sim is None:
+            if isinstance(sims, cvs.Sim):
+                base_sim = sims
+                sims = None
+            elif isinstance(sims, list):
+                base_sim = sims[0]
+            else:
+                errormsg = f'If base_sim is not supplied, sims must be either a sims or a list of sims, not {type(sims)}'
+                raise TypeError(errormsg)
+
+        if quantiles is None:
+            quantiles = make_metapars()['quantiles']
+
+        self.sims = sims
+        self.base_sim = base_sim
+        self.quantiles = quantiles
+        self.run_args = sc.mergedicts(kwargs)
+        self.results = None
+        self.which = None # Whether the multisim is to be reduced, combined, etc.
+
+        return
+
+
+    def __len__(self):
+        try:
+            return len(self.sims)
+        except:
+            return 0
+
+
+    def result_keys(self):
+        ''' Attempt to retrieve the results keys from the base sim '''
+        try:
+            keys = self.base_sim.result_keys()
+        except Exception as E:
+            errormsg = f'Could not retrieve result keys since base sim not accessible: {str(E)}'
+            raise ValueError(errormsg)
+        return keys
+
+
+    def run(self, *args, **kwargs):
+        '''
+        Run the actual scenarios
+
+        Args:
+            debug (bool): if True, runs a single run instead of multiple, which makes debugging easier
+            verbose (int): level of detail to print, passed to sim.run()
+            kwargs (dict): passed to multi_run() and thence to sim.run()
+
+        Returns:
+            None (modifies MultiSim object in place)
+        '''
+        if self.sims is None:
+            sims = self.base_sim
+        else:
+            sims = self.sims
+
+        kwargs = sc.mergedicts(self.run_args, kwargs)
+        self.sims = multi_run(sims, *args, **kwargs)
+        return
+
+
+    def combine(self, output=False):
+        ''' Combine multiple sims into a single sim with scaled results '''
+
+        n_runs = len(self)
+        combined_sim = sc.dcp(self.sims[0])
+        combined_sim.parallelized = {'parallelized':True, 'combined':True, 'n_runs':n_runs}  # Store how this was parallelized
+        combined_sim['pop_size'] *= n_runs  # Record the number of people
+
+        for s,sim in enumerate(self.sims[1:]): # Skip the first one
+            if combined_sim.people:
+                combined_sim.people += sim.people
+            for key in sim.result_keys():
+                this_res = sim.results[key]
+                combined_sim.results[key].values += this_res.values
+
+        # For non-count results (scale=False), rescale them
+        for key in combined_sim.result_keys():
+            if not combined_sim.results[key].scale:
+                combined_sim.results[key].values /= n_runs
+
+        self.base_sim = combined_sim
+        self.results = combined_sim.results
+        self.which = 'combined'
+
+        if output:
+            return self.base_sim
+        else:
+            return
+
+
+    def reduce(self, quantiles=None, output=False):
+        ''' Combine multiple sims into a single sim with scaled results '''
+
+        if quantiles is None:
+            quantiles = self.quantiles
+        if not isinstance(quantiles, dict):
+            try:
+                quantiles = {'low':quantiles[0], 'high':quantiles[1]}
+            except Exception as E:
+                errormsg = f'Could not figure out how to convert {quantiles} into a quantiles object: must be a dict with keys low, high or a 2-element array ({str(E)})'
+                raise ValueError(errormsg)
+
+        # Store information on the sims
+        n_runs = len(self)
+        reduced_sim = sc.dcp(self.sims[0])
+        reduced_sim.parallelized = {'parallelized':True, 'combined':False, 'n_runs':n_runs}  # Store how this was parallelized
+
+        # Perform the statistics
+        raw = {}
+        reskeys = reduced_sim.result_keys()
+        for reskey in reskeys:
+            raw[reskey] = np.zeros((reduced_sim.npts, len(self.sims)))
+            for s,sim in enumerate(self.sims):
+                raw[reskey][:,s] = sim.results[reskey].values
+
+        for reskey in reskeys:
+            reduced_sim.results[reskey].values[:] = np.median(raw[reskey], axis=1) # Changed from median to mean for smoother plots
+            reduced_sim.results[reskey].low       = np.quantile(raw[reskey], q=quantiles['low'],  axis=1)
+            reduced_sim.results[reskey].high      = np.quantile(raw[reskey], q=quantiles['high'], axis=1)
+        reduced_sim.likelihood() # Recompute the likelihood for the average sim
+        reduced_sim.summary_stats(verbose=False) # Recalculate the summary stats
+
+        self.base_sim = reduced_sim
+        self.results = reduced_sim.results
+        self.which = 'reduced'
+
+        if output:
+            return self.base_sim
+        else:
+            return
+
+
+    def compare(self, t=-1, sim_inds=None, output=False):
+        '''
+        Create a dataframe compare sims at a single point in time.
+
+        Args:
+            t (int or str): the day (or date) to do the comparison; default, the end
+            sim_inds (list): list of integers of which sims to include (default: all)
+            output (bool): whether or not to return the comparison as a dataframe
+
+        Returns:
+            df (dataframe): a dataframe comparison
+        '''
+
+        # Handle the indices
+        if sim_inds is None:
+            sim_inds = list(range(len(self.sims)))
+
+        # Move the results to a dictionary
+        resdict = defaultdict(dict)
+        for i,s in enumerate(sim_inds):
+            sim = self.sims[s]
+            label = sim.label
+            if not label:
+                label = f'Sim {i}'
+            for reskey in sim.result_keys():
+                val = sim.results[reskey].values[t]
+                if reskey not in ['r_eff', 'doubling_time']:
+                    val = int(val)
+                resdict[label][reskey] = val
+
+        # Convert to a dataframe
+        df = pd.DataFrame.from_dict(resdict).astype(object) # astype is necessary to prevent type coersion
+        if output:
+            return df
+        else:
+            print(df)
+            return None
+
+
+    def plot(self, *args, **kwargs):
+        ''' Convenience mthod for plotting '''
+        if self.which in ['combined', 'reduced']:
+            fig = self.base_sim.plot(*args, **kwargs)
+        else:
+            fig = None
+            orig_setylim = kwargs.get('setylim', True)
+            kwargs['legend_args'] = sc.mergedicts({'show_legend':True}, kwargs.get('legend_args')) # Only plot the legend the first time
+            for s,sim in enumerate(self.sims):
+                if s != 0:
+                    kwargs['legend_args']['show_legend'] = False
+                if s == len(self.sims)-1:
+                    kwargs['setylim'] = orig_setylim
+                else:
+                    kwargs['setylim'] = False
+                fig = sim.plot(fig=fig, *args, **kwargs)
+        return fig
+
+
+    def save(self, filename=None, keep_people=False, **kwargs):
+        '''
+        Save to disk as a gzipped pickle. Load with cv.load(filename).
+
+        Args:
+            filename (str or None): the name or path of the file to save to; if None, uses default
+            keep_people (bool): whether or not to store the population in the Sim objects (NB, very large)
+            keywords: passed to makefilepath()
+
+        Returns:
+            scenfile (str): the validated absolute path to the saved file
+
+        **Example**::
+
+            msim.save() # Saves to an .msim file
+        '''
+        if filename is None:
+            filename = 'covasim.msim'
+        scenfile = sc.makefilepath(filename=filename, **kwargs)
+        self.filename = filename # Store the actual saved filename
+
+        # Store sims seperately
+        sims = self.sims
+        self.sims = None # Remove for now
+
+        obj = sc.dcp(self) # This should be quick once we've removed the sims
+        if keep_people:
+            obj.sims = sims # Just restore the object in full
+            print('Note: saving people, which may produce a large file!')
+        else:
+            obj.base_sim.shrink(in_place=True)
+            obj.sims = []
+            for sim in sims:
+                obj.sims.append(sim.shrink(in_place=False))
+
+        sc.saveobj(filename=scenfile, obj=obj) # Actually save
+
+        self.sims = sims # Restore
+        return scenfile
+
+
 class Scenarios(cvb.ParsObj):
     '''
     Class for running multiple sets of multiple simulations -- e.g., scenarios.
@@ -202,118 +471,46 @@ class Scenarios(cvb.ParsObj):
         return
 
 
-    def plot(self, to_plot=None, do_save=None, fig_path=None, fig_args=None, plot_args=None,
-             axis_args=None, fill_args=None, legend_args=None, as_dates=True, dateformat=None,
-             interval=None, n_cols=1, font_size=18, font_family=None, grid=True, commaticks=True,
-             do_show=True, sep_figs=False, verbose=None):
+    def plot(self, *args, **kwargs):
         '''
-        Plot the results -- can supply arguments for both the figure and the plots.
+        Plot the results of a scenario.
 
         Args:
-            to_plot     (dict): Dict of results to plot; see get_scen_plots() for structure
-            do_save     (bool): Whether or not to save the figure
-            fig_path    (str):  Path to save the figure
-            fig_args    (dict): Dictionary of kwargs to be passed to pl.figure()
-            plot_args   (dict): Dictionary of kwargs to be passed to pl.plot()
-            axis_args   (dict): Dictionary of kwargs to be passed to pl.subplots_adjust()
-            fill_args   (dict): Dictionary of kwargs to be passed to pl.fill_between()
-            legend_args (dict): Dictionary of kwargs to be passed to pl.legend()
-            as_dates    (bool): Whether to plot the x-axis as dates or time points
-            dateformat  (str):  Date string format, e.g. '%B %d'
-            interval    (int):  Interval between tick marks
-            n_cols      (int):  Number of columns of subpanels to use for subplot
-            font_size   (int):  Size of the font
-            font_family (str):  Font face
-            grid        (bool): Whether or not to plot gridlines
-            commaticks  (bool): Plot y-axis with commas rather than scientific notation
-            do_show     (bool): Whether or not to show the figure
-            sep_figs    (bool): Whether to show separate figures for different results instead of subplots
-            verbose     (bool): Display a bit of extra information
+            to_plot      (dict): Dict of results to plot; see get_scen_plots() for structure
+            do_save      (bool): Whether or not to save the figure
+            fig_path     (str):  Path to save the figure
+            fig_args     (dict): Dictionary of kwargs to be passed to pl.figure()
+            plot_args    (dict): Dictionary of kwargs to be passed to pl.plot()
+            scatter_args (dict): Dictionary of kwargs to be passed to pl.scatter()
+            axis_args    (dict): Dictionary of kwargs to be passed to pl.subplots_adjust()
+            fill_args    (dict): Dictionary of kwargs to be passed to pl.fill_between()
+            legend_args  (dict): Dictionary of kwargs to be passed to pl.legend()
+            as_dates     (bool): Whether to plot the x-axis as dates or time points
+            dateformat   (str):  Date string format, e.g. '%B %d'
+            interval     (int):  Interval between tick marks
+            n_cols       (int):  Number of columns of subpanels to use for subplot
+            font_size    (int):  Size of the font
+            font_family  (str):  Font face
+            grid         (bool): Whether or not to plot gridlines
+            commaticks   (bool): Plot y-axis with commas rather than scientific notation
+            setylim      (bool): Reset the y limit to start at 0
+            log_scale    (bool): Whether or not to plot the y-axis with a log scale; if a list, panels to show as log
+            do_show      (bool): Whether or not to show the figure
+            colors       (dict): Custom color for each scenario, must be a dictionary with one entry per scenario key
+            sep_figs     (bool): Whether to show separate figures for different results instead of subplots
+            fig          (fig):  Existing figure to plot into
 
         Returns:
             fig: Figure handle
+
+
+        **Example**::
+
+            scens = cv.Scenarios()
+            scens.run()
+            scens.plot()
         '''
-
-        if verbose is None:
-            verbose = self['verbose']
-        sc.printv('Plotting...', 1, verbose)
-
-        if to_plot is None:
-            to_plot = cvd.get_scen_plots()
-        to_plot = sc.dcp(to_plot) # In case it's supplied as a dict
-
-        # Handle input arguments -- merge user input with defaults
-        fig_args    = sc.mergedicts({'figsize': (16, 14)}, fig_args)
-        plot_args   = sc.mergedicts({'lw': 3, 'alpha': 0.7}, plot_args)
-        axis_args   = sc.mergedicts({'left': 0.10, 'bottom': 0.05, 'right': 0.95, 'top': 0.90, 'wspace': 0.25, 'hspace': 0.25}, axis_args)
-        fill_args   = sc.mergedicts({'alpha': 0.2}, fill_args)
-        legend_args = sc.mergedicts({'loc': 'best'}, legend_args)
-
-        if sep_figs:
-            figs = []
-        else:
-            fig = pl.figure(**fig_args)
-        pl.subplots_adjust(**axis_args)
-        pl.rcParams['font.size'] = font_size
-        if font_family:
-            pl.rcParams['font.family'] = font_family
-
-        n_rows = np.ceil(len(to_plot)/n_cols) # Number of subplot rows to have
-        for rk,reskey in enumerate(to_plot):
-            title = self.base_sim.results[reskey].name # Get the name of this result from the base simulation
-            if sep_figs:
-                figs.append(pl.figure(**fig_args))
-                ax = pl.subplot(111)
-            else:
-                if rk == 0:
-                    ax = pl.subplot(n_rows, n_cols, rk + 1)
-                else:
-                    ax = pl.subplot(n_rows, n_cols, rk + 1, sharex=ax)
-
-            resdata = self.results[reskey]
-
-            for scenkey, scendata in resdata.items():
-
-                pl.fill_between(self.tvec, scendata.low, scendata.high, **fill_args)
-                pl.plot(self.tvec, scendata.best, label=scendata.name, **plot_args)
-                pl.title(title)
-                if rk == 0:
-                    pl.legend(**legend_args)
-
-                pl.grid(grid)
-                if commaticks:
-                    sc.commaticks()
-
-                if self.base_sim.data is not None and reskey in self.base_sim.data:
-                    data_t = np.array((self.base_sim.data.index-self.base_sim['start_day'])/np.timedelta64(1,'D'))
-                    pl.plot(data_t, self.base_sim.data[reskey], 'sk', **plot_args)
-
-                # Optionally reset tick marks (useful for e.g. plotting weeks/months)
-                if interval:
-                    xmin,xmax = ax.get_xlim()
-                    ax.set_xticks(pl.arange(xmin, xmax+1, interval))
-
-                # Set xticks as dates
-                if as_dates:
-                    @ticker.FuncFormatter
-                    def date_formatter(x, pos):
-                        return (self.base_sim['start_day'] + dt.timedelta(days=x)).strftime('%b-%d')
-                    ax.xaxis.set_major_formatter(date_formatter)
-                    if not interval:
-                        ax.xaxis.set_major_locator(ticker.MaxNLocator(integer=True))
-
-        # Ensure the figure actually renders or saves
-        if do_save:
-            if fig_path is None: # No figpath provided - see whether do_save is a figpath
-                fig_path = 'covasim_scenarios.png' # Just give it a default name
-            fig_path = sc.makefilepath(fig_path) # Ensure it's valid, including creating the folder
-            pl.savefig(fig_path)
-
-        if do_show:
-            pl.show()
-        else:
-            pl.close(fig)
-
+        fig = cvplt.plot_scens(scens=self, *args, **kwargs)
         return fig
 
 
@@ -445,7 +642,7 @@ class Scenarios(cvb.ParsObj):
         return scens
 
 
-def single_run(sim, ind=0, noise=0.0, noisepar=None, verbose=None, keep_people=False, run_args=None, sim_args=None, **kwargs):
+def single_run(sim, ind=0, reseed=True, noise=0.0, noisepar=None, verbose=None, keep_people=False, run_args=None, sim_args=None, **kwargs):
     '''
     Convenience function to perform a single simulation run. Mostly used for
     parallelization, but can also be used directly.
@@ -453,6 +650,7 @@ def single_run(sim, ind=0, noise=0.0, noisepar=None, verbose=None, keep_people=F
     Args:
         sim (Sim): the sim instance to be run
         ind (int): the index of this sim
+        reseed (bool): whether or not to generate a fresh seed for each run
         noise (float): the amount of noise to add to each run
         noisepar (string): the name of the parameter to add noise to
         verbose (int): detail to print
@@ -473,13 +671,17 @@ def single_run(sim, ind=0, noise=0.0, noisepar=None, verbose=None, keep_people=F
     new_sim = sc.dcp(sim) # Copy the sim to avoid overwriting it
 
     # Set sim and run arguments
-    if verbose is None:
-        verbose = new_sim['verbose']
     sim_args = sc.mergedicts(sim_args, kwargs)
     run_args = sc.mergedicts({'verbose':verbose}, run_args)
+    if verbose is None:
+        verbose = new_sim['verbose']
 
-    new_sim['rand_seed'] += ind # Reset the seed, otherwise no point of parallel runs
-    new_sim.set_seed()
+    if not new_sim.label:
+        new_sim.label = f'Sim {ind:d}'
+
+    if reseed:
+        new_sim['rand_seed'] += ind # Reset the seed, otherwise no point of parallel runs
+        new_sim.set_seed()
 
     # If the noise parameter is not found, guess what it should be
     if noisepar is None:
@@ -518,7 +720,7 @@ def single_run(sim, ind=0, noise=0.0, noisepar=None, verbose=None, keep_people=F
     return new_sim
 
 
-def multi_run(sim, n_runs=4, noise=0.0, noisepar=None, iterpars=None, verbose=None, combine=False, keep_people=None, run_args=None, sim_args=None, **kwargs):
+def multi_run(sim, n_runs=4, reseed=True, noise=0.0, noisepar=None, iterpars=None, verbose=None, combine=False, keep_people=None, run_args=None, sim_args=None, par_args=None, **kwargs):
     '''
     For running multiple runs in parallel. If the first argument is a list of sims,
     exactly these will be run and most other arguments will be ignored.
@@ -526,6 +728,7 @@ def multi_run(sim, n_runs=4, noise=0.0, noisepar=None, iterpars=None, verbose=No
     Args:
         sim (Sim or list): the sim instance to be run, or a list of sims.
         n_runs (int): the number of parallel runs
+        reseed (bool): whether or not to generate a fresh seed for each run
         noise (float): the amount of noise to add to each run
         noisepar (string): the name of the parameter to add noise to
         iterpars (dict): any other parameters to iterate over the runs; see sc.parallelize() for syntax
@@ -534,6 +737,7 @@ def multi_run(sim, n_runs=4, noise=0.0, noisepar=None, iterpars=None, verbose=No
         keep_people (bool): whether or not to keep the people in each sim
         run_args (dict): arguments passed to sim.run()
         sim_args (dict): extra parameters to pass to the sim
+        par_args (dict): arguments passed to sc.parallelize()
         kwargs (dict): also passed to the sim
 
     Returns:
@@ -547,9 +751,9 @@ def multi_run(sim, n_runs=4, noise=0.0, noisepar=None, iterpars=None, verbose=No
         sims = cv.multi_run(sim, n_runs=6, noise=0.2)
     '''
 
-    # Create the sims
-    if sim_args is None:
-        sim_args = {}
+    # Handle inputs
+    sim_args = sc.mergedicts(sim_args, kwargs) # Handle blank
+    par_args = sc.mergedicts(par_args) # Handle blank
 
     # Handle iterpars
     if iterpars is None:
@@ -567,36 +771,14 @@ def multi_run(sim, n_runs=4, noise=0.0, noisepar=None, iterpars=None, verbose=No
     if isinstance(sim, cvs.Sim): # Normal case: one sim
         iterkwargs = {'ind':np.arange(n_runs)}
         iterkwargs.update(iterpars)
-        kwargs = {'sim':sim, 'noise':noise, 'noisepar':noisepar, 'verbose':verbose, 'keep_people':keep_people, 'sim_args':sim_args, 'run_args':run_args}
-        sims = sc.parallelize(single_run, iterkwargs=iterkwargs, kwargs=kwargs)
+        kwargs = dict(sim=sim, reseed=reseed, noise=noise, noisepar=noisepar, verbose=verbose, keep_people=keep_people, sim_args=sim_args, run_args=run_args)
+        sims = sc.parallelize(single_run, iterkwargs=iterkwargs, kwargs=kwargs, **par_args)
     elif isinstance(sim, list): # List of sims
         iterkwargs = {'sim':sim}
-        kwargs = {'verbose':verbose, 'keep_people':keep_people, 'sim_args':sim_args, 'run_args':run_args}
-        sims = sc.parallelize(single_run, iterkwargs=iterkwargs, kwargs=kwargs)
+        kwargs = dict(verbose=verbose, keep_people=keep_people, sim_args=sim_args, run_args=run_args)
+        sims = sc.parallelize(single_run, iterkwargs=iterkwargs, kwargs=kwargs, **par_args)
     else:
         errormsg = f'Must be Sim object or list, not {type(sim)}'
         raise TypeError(errormsg)
 
-    # Usual case -- return a list of sims
-    if not combine:
-        return sims
-
-    # Or, combine them into a single sim with scaled results
-    else:
-        output_sim = sc.dcp(sims[0])
-        output_sim.parallelized = {'parallelized':True, 'combined':True, 'n_runs':n_runs}  # Store how this was parallelized
-        output_sim['pop_size'] *= n_runs  # Record the number of people
-
-        for s,sim in enumerate(sims[1:]): # Skip the first one
-            if keep_people:
-                output_sim.people += sim.people
-            for key in sim.result_keys():
-                this_res = sim.results[key]
-                output_sim.results[key].values += this_res.values
-
-        # For non-count results (scale=False), rescale them
-        for key in output_sim.result_keys():
-            if not output_sim.results[key].scale:
-                output_sim.results[key].values /= len(sims)
-
-        return output_sim
+    return sims
